package com.zkqy.web.controller.mobilepage;

import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;

import com.github.pagehelper.PageInfo;
import com.zkqy.common.utils.CollectionUtil;
import com.zkqy.common.utils.StringUtils;
import com.zkqy.system.entity.dto.MobilePageDesignDataNormalDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.zkqy.common.annotation.Log;
import com.zkqy.common.core.controller.BaseController;
import com.zkqy.common.core.domain.AjaxResult;
import com.zkqy.common.enums.BusinessType;
import com.zkqy.system.entity.MobilePageDesignData;
import com.zkqy.system.service.IMobilePageDesignDataService;
import com.zkqy.common.utils.poi.ExcelUtil;
import com.zkqy.common.core.page.TableDataInfo;

/**
 * 页面设计Controller
 *
 * @author zkqy
 * @date 2025-03-24
 */
@RestController
@RequestMapping("/system/mobilePageDesignData")
@Api(value = "/system/mobilePageDesignData", description = "页面设计-接口")
public class MobilePageDesignDataController extends BaseController
{
    @Autowired
    private IMobilePageDesignDataService mobilePageDesignDataService;

    /**
     * 查询页面设计列表
     */
    //@PreAuthorize("@ss.hasPermi('system:mobilePageDesignData:list')")
    @GetMapping("/list")
    @ApiOperation(value = "查询页面设计列表")
    public TableDataInfo list(MobilePageDesignData mobilePageDesignData)
    {
        startPage();
        List<MobilePageDesignData> list = mobilePageDesignDataService.selectMobilePageDesignDataList(mobilePageDesignData);
        return getDataTable(list);
    }

    // 查询出所有的页面信息 
    @GetMapping("/list/all")
    @ApiOperation(value = "查询页面设计列表")
    public TableDataInfo listAll()
    {
        MobilePageDesignData mobilePageDesignData = new MobilePageDesignData();
        List<MobilePageDesignData> list = mobilePageDesignDataService.selectMobilePageDesignDataList(mobilePageDesignData);
        return getDataTable(list);
    }


    /**
     * 导出页面设计列表
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageDesignData:export')")
    @Log(title = "页面设计", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    @ApiOperation(value = "导出页面设计列表")
    public void export(HttpServletResponse response, MobilePageDesignData mobilePageDesignData)
    {
        List<MobilePageDesignData> list = mobilePageDesignDataService.selectMobilePageDesignDataList(mobilePageDesignData);
        ExcelUtil<MobilePageDesignData> util = new ExcelUtil<MobilePageDesignData>(MobilePageDesignData.class);
        util.exportExcel(response, list, "页面设计数据");
    }

    /**
     * 获取页面设计详细信息
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageDesignData:query')")
    @GetMapping(value = "/{id}")
    @ApiOperation(value = "获取页面设计详细信息")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(mobilePageDesignDataService.selectMobilePageDesignDataById(id));
    }

    /**
     * 新增页面设计
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageDesignData:add')")
    @Log(title = "页面设计", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation(value = "新增页面设计")
    public AjaxResult add(@RequestBody MobilePageDesignData mobilePageDesignData)
    {
        int i = mobilePageDesignDataService.insertMobilePageDesignData(mobilePageDesignData);
        //在pageOption和pageHtml中需要把id放进去
        if (i > 0){
            mobilePageDesignDataService.fillPageIdToHtmlData(mobilePageDesignData);
            mobilePageDesignDataService.updateMobilePageDesignData(mobilePageDesignData);
        }
        return toAjax(i);
    }

    /**
     * 修改页面设计
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageDesignData:edit')")
    @Log(title = "页面设计", businessType = BusinessType.UPDATE)
    @PutMapping
    @ApiOperation(value = "修改页面设计")
    public AjaxResult edit(@RequestBody MobilePageDesignData mobilePageDesignData)
    {
        return toAjax(mobilePageDesignDataService.updateMobilePageDesignData(mobilePageDesignData));
    }

    /**
     * 删除页面设计
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageDesignData:remove')")
    @Log(title = "页面设计", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    @ApiOperation(value = "删除页面设计")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(mobilePageDesignDataService.deleteMobilePageDesignDataByIds(ids));
    }

    /**
     * 移动端 获取表格页面的数据(不对结果封装),界面预览中使用
     * 入参 表名 需要分页的参数
     */
    @GetMapping(value = "/muti/tableLimitInfo")
    @ApiOperation(value = "获取表格页面的数据")
    public TableDataInfo getMutiInfoLimit(Long pageId){
        if (pageId == 0l || pageId == null){
            return new TableDataInfo();
        }
        MobilePageDesignData mobilePageDesignData = mobilePageDesignDataService.selectMobilePageDesignDataById(pageId);
        startPage();
        String sql = mobilePageDesignDataService.mapToQuerySql(mobilePageDesignData.getPageJson());
        List<Map<String,Object>> list = mobilePageDesignDataService.executeQuerySql(sql);
        List<Map<String, Object>> underlineList = CollectionUtil.copyListMapWithCamelToUnderline(list);
        int total = new Long(new PageInfo(list).getTotal()).intValue();
        return new TableDataInfo(underlineList,total,200,"获取成功");
    }
    /**
     * 移动端 点击新增跳转的接口
     * 入参 需要跳转的页面的页面id
     * 返回值 需要跳转页面的html数据
     */
    @GetMapping("/queryMobileClickAddData/{formId}")
    public AjaxResult queryMobileClickAddData(@PathVariable("formId") Long id){
        MobilePageDesignData mobilePageDesignData = mobilePageDesignDataService.selectMobilePageDesignDataById(id);
        return success(mobilePageDesignData);
    }

    /**
     * 移动端 点击修改跳转的接口
     * 入参 表单页面的id，需要跳转到的页面id，主键id(这一行数据的主键id)
     * 返回值 需要跳转页面的html数据（查询数据并保存在html中)
     */
    @GetMapping("/queryMobileClickUpdateData/{pageId}/{formId}/{searchId}")
    public AjaxResult queryMobileClickUpdateData(@PathVariable("pageId") Long pageId,@PathVariable("formId") Long formId,@PathVariable("searchId") Long searchId){
        MobilePageDesignData fromDesignData = mobilePageDesignDataService.selectMobilePageDesignDataById(pageId);
        MobilePageDesignData toDesignData = mobilePageDesignDataService.selectMobilePageDesignDataById(formId);
        String htmlData = mobilePageDesignDataService.fillUpdateJsonPageData(fromDesignData,toDesignData,searchId);
        toDesignData.setHtmlData(htmlData);
        return success(toDesignData);
    }
    /**
     * 移动端 新增数据的接口
     * 入参 需要新增的该行数据 页面id
     * 返回值 判断是否新增成功
     */
    @PostMapping("/normal/insertData")
    public AjaxResult normalInsertData(@RequestBody MobilePageDesignDataNormalDTO mobilePageDesignDataInsertDTO){
        // 根据id获取表名或者是直接传表名 后期选一个
        Long pageId = mobilePageDesignDataInsertDTO.getPageId();
        String sql = mobilePageDesignDataService.mapToInsertSql(mobilePageDesignDataInsertDTO.getDataMap(),pageId);
        if (StringUtils.isBlank(sql)){
            return error("新增失败,请联系管理员");
        }
        return success(mobilePageDesignDataService.executeInsertSql(sql));
    }

    /**
     * 移动端 保存数据的接口
     * 入参 需要新增的该行数据
     * 返回值 判断是否保存成功
     */
    @PostMapping("/normal/updateData")
    public AjaxResult normalUpdateData(@RequestBody MobilePageDesignDataNormalDTO mobilePageDesignDataInsertDTO){
        Long pageId = mobilePageDesignDataInsertDTO.getPageId();
        Map<String, Object> dataMap = mobilePageDesignDataInsertDTO.getDataMap();
        String sql = mobilePageDesignDataService.mapToUpdateSql2(dataMap,pageId);
        return success(mobilePageDesignDataService.executeUpdateSql(sql));
    }

    /**
     * 移动端 通用删除接口
     * 入参 页面id 该行的id
     * 返回值 判断是否删除成功
     * 整体逻辑和修改差不多，因为是逻辑删除
     */
    @PostMapping("/normal/deleteData")
    public AjaxResult normalDeleteData(@RequestBody MobilePageDesignDataNormalDTO mobilePageDesignDataInsertDTO){
        // 根据id获取表名或者是直接传表名 后期选一个
        Long pageId = mobilePageDesignDataInsertDTO.getPageId();
        String sql = mobilePageDesignDataService.mapToDeleteSql(mobilePageDesignDataInsertDTO.getLineId(),pageId);
        if (StringUtils.isBlank(sql)){
            return error("删除失败,请联系管理员");
        }
        return success(mobilePageDesignDataService.executeUpdateSql(sql));
    }

}
