package com.zkqy.web.controller.mobilepage;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.zkqy.system.service.IMobilePageDesignDataService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.zkqy.common.annotation.Log;
import com.zkqy.common.core.controller.BaseController;
import com.zkqy.common.core.domain.AjaxResult;
import com.zkqy.common.enums.BusinessType;
import com.zkqy.system.entity.MobilePageNavigationBar;
import com.zkqy.system.service.IMobilePageNavigationBarService;
import com.zkqy.common.utils.poi.ExcelUtil;
import com.zkqy.common.core.page.TableDataInfo;

/**
 * 移动端页面导航条设计Controller
 *
 * @author zkqy
 * @date 2025-04-07
 */
@RestController
@RequestMapping("/system/mobilePageNavigationBar")
@Api(value = "/system/mobilePageNavigationBar", description = "移动端页面导航条设计-接口")
public class MobilePageNavigationBarController extends BaseController
{
    @Autowired
    private IMobilePageNavigationBarService mobilePageNavigationBarService;

    @Autowired
    private IMobilePageDesignDataService mobilePageDesignDataService;

    /**
     * 查询移动端页面导航条设计列表
     */
    //@PreAuthorize("@ss.hasPermi('system:mobilePageNavigationBar:list')")
    @GetMapping("/list")
    @ApiOperation(value = "查询移动端页面导航条设计列表")
    public TableDataInfo list(MobilePageNavigationBar mobilePageNavigationBar)
    {
        startPage();
        List<MobilePageNavigationBar> list = mobilePageNavigationBarService.selectMobilePageNavigationBarList(mobilePageNavigationBar);
        return getDataTable(list);
    }

    /**
     *
     * 1.查询当前用户移动端全部导航条数据
     * 2.导航条对应的表单的html页面(里面要对表格的内容进行额外的加载处理)
     */
    @GetMapping("/list/all")
    @ApiOperation(value = "查询移动端页面导航条设计列表")
    public AjaxResult listAll()
    {
        MobilePageNavigationBar mobilePageNavigationBar = new MobilePageNavigationBar();
        List<MobilePageNavigationBar> list = mobilePageNavigationBarService.selectMobilePageNavigationBarList(mobilePageNavigationBar);
        //检测是否有表格数据，如果有就填充
//        for (MobilePageNavigationBar pageNavigationBar : list) {
//            MobilePageDesignData filledTableComponentData = mobilePageDesignDataService.getFilledTableComponentData(pageNavigationBar.getPageId());
//            pageNavigationBar.setMobilePageDesignData(filledTableComponentData);
//        }
        return AjaxResult.success(list);
    }
    /**
     * 导出移动端页面导航条设计列表
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageNavigationBar:export')")
    @Log(title = "移动端页面导航条设计", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    @ApiOperation(value = "导出移动端页面导航条设计列表")
    public void export(HttpServletResponse response, MobilePageNavigationBar mobilePageNavigationBar)
    {
        List<MobilePageNavigationBar> list = mobilePageNavigationBarService.selectMobilePageNavigationBarList(mobilePageNavigationBar);
        ExcelUtil<MobilePageNavigationBar> util = new ExcelUtil<MobilePageNavigationBar>(MobilePageNavigationBar.class);
        util.exportExcel(response, list, "移动端页面导航条设计数据");
    }

    /**
     * 获取移动端页面导航条设计详细信息
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageNavigationBar:query')")
    @GetMapping(value = "/{id}")
    @ApiOperation(value = "获取移动端页面导航条设计详细信息")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(mobilePageNavigationBarService.selectMobilePageNavigationBarById(id));
    }

    /**
     * 新增移动端页面导航条设计
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageNavigationBar:add')")
    @Log(title = "移动端页面导航条设计", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation(value = "新增移动端页面导航条设计")
    public AjaxResult add(@RequestBody MobilePageNavigationBar mobilePageNavigationBar)
    {
        return toAjax(mobilePageNavigationBarService.insertMobilePageNavigationBar(mobilePageNavigationBar));
    }

    /**
     * 修改移动端页面导航条设计
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageNavigationBar:edit')")
    @Log(title = "移动端页面导航条设计", businessType = BusinessType.UPDATE)
    @PutMapping
    @ApiOperation(value = "修改移动端页面导航条设计")
    public AjaxResult edit(@RequestBody MobilePageNavigationBar mobilePageNavigationBar)
    {
        return toAjax(mobilePageNavigationBarService.updateMobilePageNavigationBar(mobilePageNavigationBar));
    }

    /**
     * 删除移动端页面导航条设计
     */
//    @PreAuthorize("@ss.hasPermi('system:mobilePageNavigationBar:remove')")
    @Log(title = "移动端页面导航条设计", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    @ApiOperation(value = "删除移动端页面导航条设计")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(mobilePageNavigationBarService.deleteMobilePageNavigationBarByIds(ids));
    }
}
