package com.zkqy.common.utils;

import lombok.Builder;
import lombok.Data;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 由client端同步过来，最新同步日期 2025/4/14
 */
@Data
public class ProcessingSelectSqlUtil {

    private List<String> columns; //所有的列名

    private String labelColumn; //label列
    private String valueColumn; //value列
    private String tableName;   //表名
    private String querySql; //后续的查询条件sql

    @Builder
    public ProcessingSelectSqlUtil(List<String> columns, String labelColumn, String valueColumn, String tableName) {
        this.columns = columns;
        this.labelColumn = labelColumn;
        this.valueColumn = valueColumn;
        this.tableName = tableName;
    }
    @Builder
    public ProcessingSelectSqlUtil(List<String> columns, String labelColumn, String valueColumn, String tableName,String querySql) {
        this.columns = columns;
        this.labelColumn = labelColumn;
        this.valueColumn = valueColumn;
        this.tableName = tableName;
        this.querySql = querySql;
    }
    @Builder
    public ProcessingSelectSqlUtil(String tableName) {
        this.tableName = tableName;
    }



    /**
     * 处理 label  和  value 中间拼接
     * @return
     */
    public  String getSelectLabelValue(){
        return columns.stream().map(
                column -> {
                    if (column.equals(valueColumn)) {
                        System.out.println("Matched valueColumn: " + column);
                        return column+","+ column + " as value";
                    }
                    if (column.equals(labelColumn)) {
                        System.out.println("Matched labelColumn: " + column);
                        return column+","+ column + " as label";
                    }
                    return column;
                }
        ).collect(Collectors.joining(","));
    };


    /**
     * 处理别名拼接
     * @return
     */
    public   String getSelectAll(){
        return columns.stream().map(
                column -> new StringBuilder(column).append(" as ")
                        .append(tableName).append("_").append(column).toString()
        ).collect(Collectors.joining(","));
    };


    /**
     * 构建完整的SQL查询
     * @return 完整的 SQL 查询字符串
     */
    public String buildFullLabelValueQuery() {
        String selectPart = getSelectLabelValue();
        return new StringBuilder("SELECT ")
                .append(selectPart)
                .append(" FROM ")
                .append("{DBNAME}.")
                .append(tableName)
                .toString();
    }

    /**
     * 构建完整的SQL查询
     * @return 完整的 SQL 查询字符串
     */
    public String buildFullAllQuery() {
        String selectPart = getSelectAll();
        return new StringBuilder("SELECT ")
                .append(selectPart)
                .append(" FROM ")
                .append(tableName)
                .toString();
    }

    public String buildFullAllNoAs() {
        //这里不用 as 作为转换
        StringBuilder selectPartBuilder = new StringBuilder();
        for (int i = 0; i < columns.size(); i++) {
            selectPartBuilder.append(columns.get(i));
            if (i < columns.size() - 1) {
                selectPartBuilder.append(",");
            }
        }
        String selectPart = selectPartBuilder.toString();
        // 入参有问题
        if (selectPart.endsWith(",")){
            return "";
        }
        return new StringBuilder("SELECT ")
                .append(selectPart)
                .append(" FROM ")
                .append(tableName)
                .toString();
    }
    //这里直接返回下划线的返回结果就可以了
    public String buildFullAllQueryWithQuerySQL() {
        return new StringBuilder(buildFullAllNoAs())
                .append(" where ").append(querySql)
                .toString();
    }
    public String buildFullSelectAllQuerySql(){
        return new StringBuilder("select *")
                .append(" FROM  ")
                .append(tableName)
                .toString();
    }
}
