package com.zkqy.system.service.impl;

import java.util.List;
import com.zkqy.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.zkqy.system.mapper.MobilePageNavigationBarMapper;
import com.zkqy.system.entity.MobilePageNavigationBar;
import com.zkqy.system.service.IMobilePageNavigationBarService;

/**
 * 移动端页面导航条设计Service业务层处理
 * 
 * @author zkqy
 * @date 2025-04-07
 */
@Service
public class MobilePageNavigationBarServiceImpl implements IMobilePageNavigationBarService 
{
    @Autowired
    private MobilePageNavigationBarMapper mobilePageNavigationBarMapper;

    /**
     * 查询移动端页面导航条设计
     * 
     * @param id 移动端页面导航条设计主键
     * @return 移动端页面导航条设计
     */
    @Override
    public MobilePageNavigationBar selectMobilePageNavigationBarById(Long id)
    {
        return mobilePageNavigationBarMapper.selectMobilePageNavigationBarById(id);
    }

    /**
     * 查询移动端页面导航条设计列表
     * 
     * @param mobilePageNavigationBar 移动端页面导航条设计
     * @return 移动端页面导航条设计
     */
    @Override
    public List<MobilePageNavigationBar> selectMobilePageNavigationBarList(MobilePageNavigationBar mobilePageNavigationBar)
    {
        return mobilePageNavigationBarMapper.selectMobilePageNavigationBarList(mobilePageNavigationBar);
    }

    /**
     * 新增移动端页面导航条设计
     * 
     * @param mobilePageNavigationBar 移动端页面导航条设计
     * @return 结果
     */
    @Override
    public int insertMobilePageNavigationBar(MobilePageNavigationBar mobilePageNavigationBar)
    {
        mobilePageNavigationBar.setCreateTime(DateUtils.getNowDate());
        return mobilePageNavigationBarMapper.insertMobilePageNavigationBar(mobilePageNavigationBar);
    }

    /**
     * 修改移动端页面导航条设计
     * 
     * @param mobilePageNavigationBar 移动端页面导航条设计
     * @return 结果
     */
    @Override
    public int updateMobilePageNavigationBar(MobilePageNavigationBar mobilePageNavigationBar)
    {
        mobilePageNavigationBar.setUpdateTime(DateUtils.getNowDate());
        return mobilePageNavigationBarMapper.updateMobilePageNavigationBar(mobilePageNavigationBar);
    }

    /**
     * 批量删除移动端页面导航条设计
     * 
     * @param ids 需要删除的移动端页面导航条设计主键
     * @return 结果
     */
    @Override
    public int deleteMobilePageNavigationBarByIds(Long[] ids)
    {
        return mobilePageNavigationBarMapper.deleteMobilePageNavigationBarByIds(ids);
    }

    /**
     * 删除移动端页面导航条设计信息
     * 
     * @param id 移动端页面导航条设计主键
     * @return 结果
     */
    @Override
    public int deleteMobilePageNavigationBarById(Long id)
    {
        return mobilePageNavigationBarMapper.deleteMobilePageNavigationBarById(id);
    }
}
